@@ -1,14 +1,32 @@
+/*
+  isp.c - main program of isp, a NXP LPC ARM controller isp programmer for Linux.
+  Copyright 2011-2013 Marc Prager
+ 
+  isp is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
+  as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
+ 
+  isp is published in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
+  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+ 
+  You should have received a copy of the GNU General Public License along with isp.
+  If not see <http://www.gnu.org/licenses/>
+ */
+
 #include <termios.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <stdio.h>
 #include <stdbool.h>
+#include <stdlib.h>
+
+#include <time.h>		// high resolution timers
+
+// c-any library headers
 #include <fifoParse.h>
 #include <fifoParseStructure.h>
 #include <fifoPopt.h>
 #include <fifoPrint.h>
 #include <lpcMemories.h>
-#include <stdlib.h>
 #include <uu.h>
 #include <macros.h>
 
@@ -16,14 +34,20 @@
 #include <c-linux/hexFile.h>
 
 /* History:
- * Version 1.0
- * ============
+ * Version 1.0 (09/2011)
+ * ======================
  * - full functionality
  *
- * Version 1.1
- * ============
+ * Version 1.1 (04/2012)
+ * ======================
  * - added Intel hex-file support, although in a not very clean programming style.
+ * - put under GPL (03/2013)
+ *
+ * Version 1.2 (07/2013)
+ * ======================
+ * - added support for LPC800
  */
+#define ISP_VERSION_COPYRIGHT "1.2 ($Rev: 62 $) (C) Marc Prager 2011-2013."
 
 static inline bool lpcReset(int fd, bool active) {
 	return serialSetDtr(fd,active);
@@ -33,10 +57,11 @@ static inline bool lpcBsl(int fd, bool active) {
 	return serialSetRts(fd,active);
 }
 
-static char outBuffer[100];
-static char inBuffer[100];
+static char outBuffer[1000];
+static char inBuffer[1000];
 Fifo fifoOut = { outBuffer, sizeof outBuffer };
 Fifo fifoIn = { inBuffer, sizeof inBuffer };
+static bool useEcho = false;
 
 bool fifoScanExactString(Fifo *input, const char *word) {
 	int length = strlen(word);
@@ -144,15 +169,10 @@ const char* returnMessage(int returnCode) {
 	}
 }
 
-int lpc17Id2PartNumber(int id) {
-	LpcFamilyMember const *member = lpcFindById(lpcFamilyMembers17xx,id);
-	if (member!=0) return member->number;
-	else return 0;
-}
-
 bool ioScanLine(int fd, Fifo *line) {
 	char c;
 	while (1==read(fd,&c,1)) {
+		fprintf(stderr,"{%c=0x%02X}",c,c & 0xFF ); fflush(stderr);
 		switch(c) {
 			case 0x13:	// flow control: stop
 			case 0x11:	// flow control: start
@@ -169,6 +189,12 @@ bool ioScanLine(int fd, Fifo *line) {
 
 int ioScanResult(int fd) {
 	Fifo line;
+	if (useEcho && (!ioScanLine(fd,&line))) {
+		fprintf(stderr,"Input buffer dump [%lu]:",fifoCanRead(&fifoIn));
+		fifoDump(&fifoIn);
+		return -1;
+	}
+
 	if (ioScanLine(fd,&line)) {
 		int returnCode;
 		if (fifoParseInt(&line,&returnCode)) {
@@ -339,7 +365,7 @@ bool lpcBailOut(int fd) {
 	return false;
 }
 
-bool lpcRead(int fd, Fifo *data, unsigned address, int n) {
+bool lpcReadUuencode(int fd, Fifo *data, unsigned address, int n) {
 	const int r0 = fifoCanRead(data);
 
 	if (lpcCommandIntInt(fd,'R',address,n)) {
@@ -368,7 +394,47 @@ bool lpcRead(int fd, Fifo *data, unsigned address, int n) {
 	else return false;
 }
 
-bool lpcWrite(int fd, unsigned address, Fifo *data) {
+bool lpcReadBinary(int fd, Fifo *data, unsigned address, int n) {
+	//const int r0 = fifoCanRead(data);
+
+	if (lpcCommandIntInt(fd,'R',address,n)) {
+		// :o) difficult mode switching line-oriented <=> binary
+	/*
+		for (int b=0; fifoCanRead(data)-r0 <n; lineNo++) {
+			Fifo fifoLine;
+			if (ioScanLine(fd,&fifoLine) && fifoUuDecodeLine(data,&fifoLine,&checksum)) ;	// fine
+			else return lpcBailOut(fd);
+
+			if ((lineNo % 20)==19 || fifoCanRead(data)-r0 ==n) {
+				int checksumLpc;
+				if (ioScanInt(fd,&checksumLpc)
+				&& checksum==checksumLpc) {
+					fprintf(stderr,".");
+					ioWriteString(fd,"OK\r\n");	// :o) unchecked
+				}
+				else {
+					fprintf(stderr,"Checksum Error.\n");
+					return lpcBailOut(fd);	// we do not retry
+				}
+				checksum = 0;
+			}
+		}
+	*/
+		return true;
+	}
+	else return false;
+}
+
+bool lpcRead(int fd, int ispData, Fifo *data, unsigned address, int n) {
+	switch(ispData) {
+		case ISP_DATA_UUENCODE: return lpcReadUuencode(fd,data,address,n);
+		case ISP_DATA_BINARY: return lpcReadBinary(fd,data,address,n);
+		default: return lpcBailOut(fd);
+	}
+}
+
+bool lpcWriteUuencode(int fd, unsigned address, Fifo *data) {
+fprintf(stderr,"lpcWriteUuencode()");
 	if (lpcCommandIntInt(fd,'W',address,fifoCanRead(data))) {
 		int checksum=0;
 		for (int lineNo=0; fifoCanRead(data); lineNo++) {
@@ -400,10 +466,31 @@ bool lpcWrite(int fd, unsigned address, Fifo *data) {
 	else return false;
 }
 
-bool lpcWriteN(int fd, unsigned address, Fifo *data, int n) {
+bool lpcWriteBinary(int fd, unsigned address, Fifo *data) {
+fprintf(stderr,"lpcWriteBinary()");
+	const int n = fifoCanRead(data);
+	if (lpcCommandIntInt(fd,'W',address,n)) {
+		fifoAppend(&fifoOut,data);
+		fprintf(stderr,".");
+		fflush(stderr);
+		return	ioWriteFifo(fd,&fifoOut) && ioScanString(fd,"OK")
+			|| lpcBailOut(fd);
+	}
+	else return false;
+}
+
+bool lpcWrite(int fd, int ispData, unsigned address, Fifo *data) {
+	switch(ispData) {
+		case ISP_DATA_UUENCODE: return lpcWriteUuencode(fd,address,data);
+		case ISP_DATA_BINARY: return lpcWriteBinary(fd,address,data);
+		default: return lpcBailOut(fd);
+	}
+}
+
+bool lpcWriteN(int fd, int ispData, unsigned address, Fifo *data, int n) {
 	Fifo fifoFraction;
 	return	fifoParseN(data,&fifoFraction,n)
-		&& lpcWrite(fd,address,&fifoFraction);
+		&& lpcWrite(fd,ispData,address,&fifoFraction);
 }
 
 typedef struct {
@@ -437,7 +524,7 @@ bool lpcProgramNew(int fd, LpcFamilyMember const *member, const HexImage *hexIma
 
 	// optimized for big images => maximum block with 'Copy RAM to FLASH' unless image is really small.
 	int blockSize = 0;
-	for (int i=0; i<4 && map->blockSizes[i]>0; ++i) {
+	for (int i=0; i<ELEMENTS(map->blockSizes) && map->blockSizes[i]>0; ++i) {
 		if (map->blockSizes[i] <= blockLimit
 		&& blockSize<imageSize) 	blockSize = map->blockSizes[i];
 	}
@@ -489,7 +576,7 @@ bool lpcProgramNew(int fd, LpcFamilyMember const *member, const HexImage *hexIma
 		// transfer bytes
 		for (int subBlock=0; subBlock<blocksAtOnce && block+subBlock<blocks; ++subBlock) {
 			fprintf(stderr,"\nTransfer block %d to RAM ",block+subBlock);
-			if (lpcWriteN(fd,addressRam+subBlock*blockSize,&fifoImage,blockSize)) {	// fine
+			if (lpcWriteN(fd,member->ispData,addressRam+subBlock*blockSize,&fifoImage,blockSize)) {	// fine
 				fprintf(stderr,"OK.");
 			}
 			else {
@@ -547,7 +634,7 @@ bool lpcProgram(int fd, LpcFamilyMember const *member, unsigned address, Fifo *d
 
 	// optimized for big images => maximum block with 'Copy RAM to FLASH' unless image is really small.
 	int blockSize = 0;
-	for (int i=0; i<4 && map->blockSizes[i]>0; ++i) {
+	for (int i=0; i<ELEMENTS(map->blockSizes) && map->blockSizes[i]>0; ++i) {
 		if (map->blockSizes[i] <= blockLimit
 		&& blockSize<imageSize) 	blockSize = map->blockSizes[i];
 	}
@@ -599,7 +686,7 @@ bool lpcProgram(int fd, LpcFamilyMember const *member, unsigned address, Fifo *d
 		// transfer bytes
 		for (int subBlock=0; subBlock<blocksAtOnce && block+subBlock<blocks; ++subBlock) {
 			fprintf(stderr,"\nTransfer block %d to RAM ",block+subBlock);
-			if (lpcWriteN(fd,addressRam+subBlock*blockSize,data,blockSize)) {	// fine
+			if (lpcWriteN(fd,member->ispData,addressRam+subBlock*blockSize,data,blockSize)) {	// fine
 				fprintf(stderr,"OK.");
 			}
 			else {
@@ -704,17 +791,19 @@ static Int32 ucNumber = -1;		// force specific device
 static int ucSymbol = -1;		// force user supplied.
 static Fifo fifoF = {};
 static Fifo fifoM = {};
+static Int32 ispProtocolOverride = -1;	// binary / uuencode override
 static PairInt32 erase = { 0, -1 };
 static Int32 ispReserved[2] = { 0x200, 0x120 };	// RAM used by ISP handler, bottom/top of RAM0
 static Int32 userId = -1;
 static Int32 userNumber = -1;
-static Int32 listB[4] = { };
+static Int32 listB[LPC_BLOCK_SIZES] = { };
 
 static const FifoPoptBool optionBools[] = {
 	{ 'i',&showInfo },
 	{ 'w',&writeAll },
 	{ 'v',&verify },
 	{ 'q',&quick },
+	{ 'u',&useEcho },
 	{}
 };
 
@@ -760,11 +849,17 @@ static const char* manualSymbols[] = {
 	0
 };
 
+static const char* ispProtocols[] = {
+	"UUENCODE","BINARY",		// these positions must match the enum im lpcMemories.h !
+	0
+};
+
 static const FifoPoptSymbol optionSymbols[] = {
 	{ 'e',&eraseSymbol, eraseSymbols },
 	{ 'h',&manualSymbol, manualSymbols },
 	{ 'x',&executeSymbol, executeSymbols },
 	{ 'U',&ucSymbol, ucSymbols },
+	{ 'P',&ispProtocolOverride, ispProtocols },
 	{}
 };
 
@@ -824,7 +919,7 @@ PairInt listM[3];
 
 static void showHelp(void) {
 	fprintf(stderr,"%s",
-	"isp 1.1 (C) Marc Prager 2011-2012\n"
+	"isp " ISP_VERSION_COPYRIGHT "\n"
 	"A command-line ISP programmer for NXP LPC1xxx,2xxx devices.\n\n"
 	"Usage: isp <options> [<inputFile>]\n"
 	"Options:\n"
@@ -846,9 +941,11 @@ static void showHelp(void) {
 	"  -s                        : show all compiled-in devices.\n"
 	"  -t <time/ms>              : time isp waits after RESET deassertion [200]\n"
 	"  -T <time/ms>              : timeout of serial communication, step: 100ms [1000]\n"
+	"  -u                        : use echo on (bug fix for LPC812)\n"
 	"  -v                        : verify FLASH contents\n"
 	"  -w                        : write to FLASH\n"
 	"  -x {<address>|LMA|LMA+1}  : execute. ARM/THUMB chosen from address LSB\n"
+	"  -P {UUENCODE|BINARY}      : data encoding protocol\n"
 	"  -R <bottom>,<top>         : set RAM0 spaces reserved for ISP [0x200,0x120]\n"
 	"  -D <device database file> : load device descriptions from file.\n"
 	"  -I <uC ID>                : set user-defined id\n"
@@ -863,6 +960,8 @@ static void showHelp(void) {
 	"If your controller does not work correctly with isp, try low baud rate, low serial\n"
 	"timeout (-T 1000 or more) and correct crystal frequency.\n"
 	"Try -h contents for online manual\n"
+	"\n"
+	"Please report bugs to marc@windscooting.com\n"
 	);
 }
 
@@ -893,7 +992,7 @@ void manualNumericArguments(void) {
 	"  -e 1..3          erases sectors 1 to 3.\n"
 	"  -e all           erases all sectors of the device in use.\n"
 	"  -B 256,512,1024,4096  defines the possible transfer sizes used to copy RAM\n"
-	"                   to flash for most (all?) LPCs. It is a 4-tuple.\n"
+	"                   to flash for most (all?) LPCs. It is a 4..5-tuple.\n"
 	"Defining the flash sector sizes is done with a list of pairs, with a bit of\n"
 	"eye-candy added, as an example LPC213x:\n"
 	"  -F 4kix8,32kix14,4kix5      as the short form of\n"
@@ -1019,6 +1118,10 @@ bool lpcCrp(const HexImage *hexImage, Uint32 addressCrp, int crpMax, int crp) {
 	return fifoLpcCrp(&fifo,addressCrp,hexImage->segments[0].address,crpMax,crp);
 }
 
+int noConnection(void) {
+	fprintf(stderr,"No connection/connection lost (try different baud rate/crystal?).\n");
+	return 1;
+}
 
 /*
 Planned features:
@@ -1031,6 +1134,11 @@ Planned features:
  - piping
 */
 int main(int argc, const char **argv) {
+	struct timespec ts;
+	//clock_getres(CLOCK_PROCESS_CPU_TIME_ID,&ts);
+	clock_getres(CLOCK_REALTIME,&ts);
+	fprintf(stderr,"Clock resolution: %lds %ldns\n",ts.tv_sec,ts.tv_nsec);
+
 	char lineBuffer[1024];
 	Fifo fifoCmdLine = { lineBuffer, sizeof lineBuffer };
 	Fifo fifoNonOptions = { lineBuffer, sizeof lineBuffer };
@@ -1119,18 +1227,42 @@ int main(int argc, const char **argv) {
 
 	int partId = 0;
 	int bootCodeVersion = 0;
-	if (lpcSync(fd,crystalHz)
-	&& lpcEcho(fd,false)
-	&& lpcBaud(fd,baud,1)
-	&& lpcReadBootCodeVersion(fd,&bootCodeVersion)
-	&& lpcReadPartId(fd,&partId)) {	// fine
-		fprintf(stderr,"Device Id=0x%08X, boot code %d.%d\n",partId,bootCodeVersion>>8,bootCodeVersion&0xFF);
+	fprintf(stderr,"Sync:"); fflush(stderr);
+	if (lpcSync(fd,crystalHz)) {
+		fprintf(stderr,"OK.");
+		fflush(stderr);
 	}
-	else {
-		fprintf(stderr,"No connection/connection lost (try different baud rate/crystal?).\n");
-		return 1;
+	else return noConnection();
+
+	if (!useEcho) {
+		fprintf(stderr,", echo off:"); fflush(stderr);
+		if (lpcEcho(fd,false)) {
+			fprintf(stderr,"OK.");
+			fflush(stderr);
+		}
+		else return noConnection();
 	}
 
+	fprintf(stderr,", baud:"); fflush(stderr);
+	if (lpcBaud(fd,baud,1)) {
+		fprintf(stderr,"OK.");
+		fflush(stderr);
+	}
+	else return noConnection();
+
+	fprintf(stderr,", boot code:" ); fflush(stderr);
+	if (lpcReadBootCodeVersion(fd,&bootCodeVersion)) {
+		fprintf(stderr,"%d.%d.",bootCodeVersion>>8,bootCodeVersion&0xFF);
+		fflush(stderr);
+	}
+	else return noConnection();
+
+	fprintf(stderr,", device Id:"); fflush(stderr);
+	if (lpcReadPartId(fd,&partId)) {	// fine
+		fprintf(stderr,"0x%08X\n",partId);
+	}
+	else return noConnection();
+
 	// calculate device
 	// user (command-line) defined device...
 	LpcFamilyMemoryMap familyMemoryMapUser = {};
@@ -1138,7 +1270,7 @@ int main(int argc, const char **argv) {
 		.familyMemoryMap = &familyMemoryMapUser
 	};
 
-	bool userDefinedAvailable = false;
+	//bool userDefinedAvailable = false;
 
 	if (nFs>=1		// option -F
 	|| nMs>=1		// option -M
@@ -1172,7 +1304,7 @@ int main(int argc, const char **argv) {
 			lpcFamilySectorHighest(&familyMemoryMapUser)+1) / 1024;
 		familyMemberUser.number = userNumber;
 		familyMemberUser.id = userId;		// -1 hopefully matches no real device. 0 neither.
-		userDefinedAvailable = true;
+		//userDefinedAvailable = true;
 	}
 	else {
 		fprintf(stderr,"Options -F, -M, -B, -N are required for defining new device.\n");
@@ -1232,6 +1364,7 @@ int main(int argc, const char **argv) {
 	// now patch the device with command line overrides
 	if (flashSizeOverride!=-1) member.sizeFlashK = flashSizeOverride/1024;
 	if (memorySizeOverride!=-1) member.sizeRamKs[0] = memorySizeOverride/1024;
+	if (ispProtocolOverride!=-1) member.ispData = ispProtocolOverride;
 
 	// show info, if desired
 	if (showInfo) {
@@ -1248,7 +1381,7 @@ int main(int argc, const char **argv) {
 		char imageBuffer[512*1024];
 		Fifo fifoImage = { imageBuffer, sizeof imageBuffer };
 
-		if (lpcRead(fd,&fifoImage,address,readN)) {
+		if (lpcRead(fd,member.ispData,&fifoImage,address,readN)) {
 			if (ioWriteFifo(1,&fifoImage)) {	// write to stdout, fd 1
 				fprintf(stderr,"FLASH read successfully.\n");
 			}
